@@ -1,6 +1,7 @@
 # coding: utf-8
 """Dragonfly Story."""
 from __future__ import division
+import math
 
 from ladybug_geometry.geometry2d.polygon import Polygon2D
 from ladybug_geometry.geometry3d.pointvector import Vector3D
@@ -13,6 +14,8 @@ from honeybee.typing import float_positive, int_in_range, clean_string, \
     invalid_dict_error
 from honeybee.boundarycondition import boundary_conditions as bcs
 from honeybee.boundarycondition import Outdoors, Surface
+from honeybee.facetype import AirBoundary
+from honeybee.facetype import face_types as ftyp
 from honeybee.altnumber import autocalculate
 from honeybee.shade import Shade
 from honeybee.room import Room
@@ -697,6 +700,29 @@ using-multipliers-zone-and-or-window.html
         for room in self.room_2ds:
             room.align(line_ray, distance)
 
+    def align(self, line_ray, distance, tolerance=0.01):
+        """Move Room2D and Roof vertices within a distance of a line to be on that line.
+
+        This method differs from the align_room_2ds method in that it will also
+        align any Roof geometry (if it is present).
+
+        Args:
+            line_ray: A ladybug_geometry Ray2D or LineSegment2D to which the Room2D
+                and Roof vertices will be aligned. Ray2Ds will be interpreted as being
+                infinite in both directions while LineSegment2Ds will be interpreted
+                as only existing between two points.
+            distance: The maximum distance between a vertex and the line_ray where
+                the vertex will be moved to lie on the line_ray. Vertices beyond
+                this distance will be left as they are.
+            tolerance: The minimum distance between vertices below which they are
+                considered co-located. This is used to ensure that the alignment process
+                does not create new overlaps in the roof geometry. (Default: 0.01,
+                suitable for objects in meters).
+        """
+        self.align_room_2ds(line_ray, distance)
+        if self.roof is not None:
+            self.roof.align(line_ray, distance, tolerance)
+
     def remove_room_2d_duplicate_vertices(self, tolerance=0.01, delete_degenerate=False):
         """Remove duplicate vertices from all Room2Ds in this Story.
 
@@ -991,13 +1017,71 @@ using-multipliers-zone-and-or-window.html
         for room in self._room_2ds:
             room.is_top_exposed = is_top_exposed
 
+    def split_with_story_above(self, story_above, tolerance=0.01):
+        """Split the child Room2Ds of this object with the footprint of the Story above.
+
+        This is useful as a pre-step before running set_top_exposed_by_story_above
+        as it ensures all top-exposed areas of this Story have a Room2D that can
+        be set to exposed.
+
+        Args:
+            story_above: A Story object that sits above this Story. Each Room2D
+                of this Story will be checked to see if it intersects the Story
+                above and it will be split based on this.
+            tolerance: The tolerance with which the splitting intersection will be
+                computed. Default: 0.01, suitable for objects in meters.
+        """
+        # get the footprint geometry of the story above
+        above_geos = story_above.footprint(tolerance)
+        # loop through the rooms and split them
+        split_rooms = []
+        for room in self._room_2ds:
+            # split the floor with all geometries above
+            room_height = room.floor_height
+            split_geo = [room.floor_geometry]
+            for i, a_geo in enumerate(above_geos):
+                # make sure all above geometries are at the room floor_height
+                if abs(a_geo[0].z - room_height) > tolerance:
+                    a_geo = a_geo.move(Vector3D(0, 0, room_height - a_geo[0].z))
+                    above_geos[i] = a_geo  # set it so we hopefully don't move next time
+                # split the geometries with one another
+                new_geo = []
+                for r_geo in split_geo:
+                    floor_split, above_split = Face3D.coplanar_split(
+                        r_geo, a_geo, tolerance, 1)
+                    new_geo.extend(floor_split)
+                split_geo = new_geo
+            # create the new Room2D if necessary
+            if len(split_geo) == 1:  # no room splitting needed
+                split_rooms.append(room)
+            else:  # the Room2D has been split
+                for j, s_geo in enumerate(split_geo):
+                    # check to make sure the split geometry is not degenerate
+                    max_dim = max((s_geo.max.x - s_geo.min.x, s_geo.max.y - s_geo.min.y))
+                    if s_geo.area < max_dim * tolerance:  # degenerate geometry found
+                        continue
+                    new_id = '{}_{}'.format(room.identifier, j)
+                    new_room = Room2D(
+                        new_id, s_geo, room.floor_to_ceiling_height,
+                        is_ground_contact=room.is_ground_contact,
+                        is_top_exposed=room.is_top_exposed)
+                    room._match_and_transfer_wall_props(new_room, tolerance)
+                    new_room._display_name = room.display_name
+                    new_room._user_data = None if room.user_data is None \
+                        else room.user_data.copy()
+                    new_room._skylight_parameters = room._skylight_parameters
+                    new_room._properties._duplicate_extension_attr(room._properties)
+                    split_rooms.append(new_room)
+        # set the split rooms to this story
+        self.room_2ds = split_rooms
+
     def set_top_exposed_by_story_above(self, story_above, tolerance=0.01):
         """Set the child Room2Ds of this object to have ceilings exposed to the outdoors.
 
         Args:
             story_above: A Story object that sits above this Story. Each Room2D
-                of this Story will be checked to see if it intersects the Story
-                above and the top exposure will be set based on this.
+                of this Story will be checked to see if the story_above geometry
+                lies above the room and, if not, the top exposure will be set to True.
             tolerance: The tolerance that will be used to compute the point within
                 the floor boundary that is used to check whether there is geometry
                 above each Room2D. It is recommended that this number not be less
@@ -1252,13 +1336,21 @@ using-multipliers-zone-and-or-window.html
                                 adj_face = adj[1][-2]
                                 if face.identifier == adj_face:
                                     self._match_apertures(adj[0], face)
-                                    try:
-                                        adj[0].set_adjacency(face, tolerance)
-                                    except (AssertionError, ValueError) as e:
-                                        if enforce_adj:
-                                            raise e
-                                        face.boundary_condition = bcs.outdoors
-                                        adj[0].boundary_condition = bcs.outdoors
+                                    other_resolve = False
+                                    if self.roof is not None:  # two roofs may meet
+                                        tol_area = math.sqrt(face.area) * tolerance
+                                        if abs(face.area - adj[0].area) > tol_area:
+                                            self._resolve_roof_adj(
+                                                face, adj[0], tolerance)
+                                            other_resolve = True
+                                    if not other_resolve:
+                                        try:
+                                            adj[0].set_adjacency(face, tolerance)
+                                        except (AssertionError, ValueError) as e:
+                                            if enforce_adj:
+                                                raise e
+                                            face.boundary_condition = bcs.outdoors
+                                            adj[0].boundary_condition = bcs.outdoors
                                     adj_set.add(face.identifier)
                                     break
                             break
@@ -1328,6 +1420,67 @@ using-multipliers-zone-and-or-window.html
             except AttributeError:
                 pass  # honeybee-energy extension is not loaded
 
+    @staticmethod
+    def _resolve_roof_adj(face_1, face_2, tol):
+        """Resolve incorrect adjacency where walls of two roofs meet."""
+        # remove air boundary conditions so the split result is valid
+        use_ab = False
+        if isinstance(face_1.type, AirBoundary) or isinstance(face_2.type, AirBoundary):
+            face_1.type = ftyp.wall
+            face_2.type = ftyp.wall
+            use_ab = True
+
+        # split the adjacent walls with one another to get a match
+        room_1, room_2 = face_1.parent, face_2.parent
+        new_faces1 = room_1.coplanar_split([face_2.geometry], tol)
+        new_faces2 = room_2.coplanar_split([face_1.geometry], tol)
+        new_faces1 = [face_1] if len(new_faces1) == 0 else new_faces1
+        new_faces2 = [face_2] if len(new_faces2) == 0 else new_faces2
+        
+        # find the adjacency and set it
+        adj_geo = None
+        for j, f_1 in enumerate(new_faces1):
+            for k, f_2 in enumerate(new_faces2):
+                if f_1.geometry.is_centered_adjacent(f_2.geometry, tol):
+                    f_1.set_adjacency(f_2)
+                    adj_geo = f_1.geometry
+                    if use_ab:
+                        f_1.type = ftyp.air_boundary
+                        f_2.type = ftyp.air_boundary
+                    new_faces1.pop(j)
+                    new_faces2.pop(k)
+                    break
+
+        # set the boundary conditions of the other newly-created Faces
+        for nf in new_faces1:
+            for of in room_2.faces:
+                if nf.geometry.is_centered_adjacent(of.geometry, tol):
+                    nf.set_adjacency(of)
+                    if use_ab:
+                        nf.type = ftyp.air_boundary
+                        of.type = ftyp.air_boundary
+                    break
+            else:
+                if adj_geo is not None:
+                    if nf.center.z < adj_geo.center.z:
+                        nf.boundary_condition = room_2[0].boundary_condition
+                    elif nf.center.z > adj_geo.center.z:
+                        nf.boundary_condition = room_2[-1].boundary_condition
+        for nf in new_faces2:
+            for of in room_1.faces:
+                if nf.geometry.is_centered_adjacent(of.geometry, tol):
+                    nf.set_adjacency(of)
+                    if use_ab:
+                        nf.type = ftyp.air_boundary
+                        of.type = ftyp.air_boundary
+                    break
+            else:
+                if adj_geo is not None:
+                    if nf.center.z < adj_geo.center.z:
+                        nf.boundary_condition = room_1[0].boundary_condition
+                    elif nf.center.z > adj_geo.center.z:
+                        nf.boundary_condition = room_1[-1].boundary_condition
+
     def __copy__(self):
         new_s = Story(
             self.identifier, tuple(room.duplicate() for room in self._room_2ds),
